@@ -350,8 +350,10 @@ namespace Tasque
 				return;
 			}
 			
-			crc.Text = 	date.ToString(Catalog.GetString("M/d - ddd"));
-			
+			if (date.Year == DateTime.Today.Year)
+				crc.Text = date.ToString(Catalog.GetString("M/d - ddd"));
+			else
+				crc.Text = date.ToString(Catalog.GetString("M/d/yy - ddd"));
 			//Utilities.GetPrettyPrintDate (task.DueDate, false);
 		}
 		
@@ -541,7 +543,23 @@ namespace Tasque
 			if (task == null)
 				return;
 			
-			task.Name = args.NewText;
+			string newText = args.NewText;
+			
+			// Attempt to derive due date information from text.
+			if (Application.Preferences.GetBool (Preferences.ParseDateEnabledKey) &&
+			    task.State == TaskState.Active &&
+			    task.DueDate == DateTime.MinValue) {
+				
+				string parsedTaskText;
+				DateTime parsedDueDate;
+				Utilities.ParseTaskText (newText, out parsedTaskText, out parsedDueDate);
+				
+				if (parsedDueDate != DateTime.MinValue)
+					task.DueDate = parsedDueDate;
+				newText = parsedTaskText;
+			}
+			
+			task.Name = newText;
 		}
 		
 		/// <summary>
@@ -556,6 +574,11 @@ namespace Tasque
 		/// </param>
 		void OnDateEdited (object sender, Gtk.EditedArgs args)
 		{
+			if (args.NewText == null) {
+				Logger.Debug ("New date text null, not setting date");
+				return;
+			}
+			
 			Gtk.TreeIter iter;
 			Gtk.TreePath path = new TreePath (args.Path);
 			if (Model.GetIter (out iter, path) == false)
@@ -712,14 +735,19 @@ namespace Tasque
 			
 			private bool PulseAnimation ()
 			{
-				// Emit this signal to cause the TreeView to update the row
-				// where the task is located.  This will allow the
-				// CellRendererPixbuf to update the icon.
-				tree.Model.EmitRowChanged (path, iter);
-				
-				// Return true so that this method will be called after an
-				// additional timeout duration has elapsed.
-				return true;
+				if (tree.Model == null) {
+					// Widget has been closed, no need to call this again
+					return false;
+				} else {
+					// Emit this signal to cause the TreeView to update the row
+					// where the task is located.  This will allow the
+					// CellRendererPixbuf to update the icon.
+					tree.Model.EmitRowChanged (path, iter);
+					
+					// Return true so that this method will be called after an
+					// additional timeout duration has elapsed.
+					return true;
+				}
 			}
 		}
 		#endregion // Private Classes
